# -*- rpm-spec -*-

# The full kernel uname version
%define uname         3.10.0+2

%ifarch %{ix86}
%define target       %{_target_cpu}
%define arch	     i386
%endif

%ifarch x86_64
%define target       %{_target_cpu}
%define arch         %{_target_cpu}
%endif

%define COMMON_MAKEOPTS %{?_smp_mflags}
%define DEFINE_MAKEOPTS \
	ARCH=%{arch}; \
	unset CROSS_COMPILE; \
 	case ${ARCH} in \
	    *64) [ $(uname -m) = ${ARCH} ] || CROSS_COMPILE=x86_64-linux- ;; \
	esac ; \
	MAKEOPTS="%{COMMON_MAKEOPTS} ARCH=${ARCH} CROSS_COMPILE=${CROSS_COMPILE}"


#
# First the general kernel required versions as per
# Documentation/Changes
#
#%define kernel_dot_org_conflicts  ppp < 2.4.0, pcmciautils < 004, isdn4k-utils < 3.1pre1, nfs-utils < 1.0.5, e2fsprogs < 1.41.4, util-linux < 2.10, jfsutils < 1.1.3, reiserfsprogs < 3.6.3, xfsprogs < 2.6.0, procps < 3.2.0, oprofile < 0.9
%define kernel_dot_org_conflicts  ppp < 2.4.0, pcmciautils < 004, isdn4k-utils < 3.1pre1, nfs-utils < 1.0.5, util-linux < 2.10, jfsutils < 1.1.3, reiserfsprogs < 3.6.3, xfsprogs < 2.6.0, procps < 3.2.0, oprofile < 0.9

#
# Then a series of requirements that are distribution specific, either
# because we add patches for something, or the older versions have
# problems with the newer kernel or lack certain things that make
# integration in the distro harder than needed.
#
%define package_conflicts  initscripts < 7.23, udev < 063-6, iptables < 1.3.2.1-1-1, ipw2200-firmware < 2.2

#
# Packages that need to be installed before the kernel is, because the %post
# scripts use them.
#
%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, /sbin/new-kernel-pkg

#
# don't use RPM's internal dependency generator, instead
# just use our magic one that finds the versions of kernel modules for
# provides and don't do any requires
# (we used to just turn off AutoReqProv on all packages)
#
%define _use_internal_dependency_generator 0
%define __find_provides %{nil}
%define __find_requires %{nil}

Name: kernel
Vendor: Citrix Systems, Inc.
Group: System Environment/Kernel
License: GPLv2
Version: 3.10.41
Release: 349.380447
ExclusiveArch: noarch i686 x86_64
ExclusiveOS: Linux
Summary: The Linux kernel built for Xen.
BuildRequires(pre): module-init-tools, patch >= 2.5.4, bash >= 2.03, sh-utils, tar
BuildRequires(pre): bzip2, findutils, gzip, m4, perl, make >= 3.78
BuildRequires: gcc >= 2.96-98, binutils >= 2.12, redhat-rpm-config >= 8.0.32.1
Provides: kernel-uname-r = %{uname}
Autoreqprov: no
Requires(pre): %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}

Source0: linux-%{version}-%{release}.tar.bz2
Source1: kernel-%{arch}.config

BuildRoot: %{_tmppath}/kernel-%{version}-root

%description
This package provides the Linux kernel for privileged and unprivileged domains.

trunk.72c0f4193873987686d776813b96ef5e17b060b0.45e05c6836e60b79ab5ac3b1738422837e2b82a3

%package headers
Summary: Header files for the Linux kernel for use by glibc
Group: Development/System
Obsoletes: glibc-kernheaders
Provides: kernel-headers = %{uname}
Conflicts: kernel-headers < %{uname}

%description headers
This package provides the C header files that specify the interface
between the Linux kernel and userspace libraries & programs. The
header files define structures and constants that are needed when
building most standard programs. They are also required when
rebuilding the glibc package.

trunk.72c0f4193873987686d776813b96ef5e17b060b0.45e05c6836e60b79ab5ac3b1738422837e2b82a3


%package devel
Summary: Development package for building kernel modules to match the xen kernel.
Group: System Environment/Kernel
AutoReqProv: no
Provides: kernel-devel = %{uname}

%description devel
This package provides kernel headers and makefiles sufficient to build modules
against the %{uname} kernel.

trunk.72c0f4193873987686d776813b96ef5e17b060b0.45e05c6836e60b79ab5ac3b1738422837e2b82a3


#%package doc
#Summary: Various documentation bits found in the kernel source.
#Group: Documentation
#
#%description doc
#This package contains documentation files from the kernel
#source. Various bits of information about the Linux kernel and the
#device drivers shipped with it are documented in these files.
#
#You'll want to install this package if you need a reference to the
#options that can be passed to Linux kernel modules at load time.

%prep 

%setup -q -n %{name}-%{version} -c
cd linux-%{version}
%{DEFINE_MAKEOPTS}
make ${MAKEOPTS} mrproper
cp -f %{_sourcedir}/kernel-%{target}.config .config

%build

cd linux-%{version}
%{DEFINE_MAKEOPTS}
make ${MAKEOPTS} silentoldconfig
if grep -q ^CONFIG_MODULES=y$ .config ; then
    mkmodules=modules
else
    mkmodules=
fi

make ${MAKEOPTS} bzImage $mkmodules

%install

cd linux-%{version}
%{DEFINE_MAKEOPTS}
# Install kernel
install -d -m 755 $RPM_BUILD_ROOT/boot
install -m 644 .config $RPM_BUILD_ROOT/boot/config-%{uname}
install -m 644 System.map $RPM_BUILD_ROOT/boot/System.map-%{uname}
install -m 644 arch/x86/boot/bzImage $RPM_BUILD_ROOT/boot/vmlinuz-%{uname}

install -d -m 755 $RPM_BUILD_ROOT/lib/modules/%{uname}
if grep -q ^CONFIG_MODULES=y$ .config ; then
    # Install modules
    make ${MAKEOPTS} INSTALL_MOD_PATH=$RPM_BUILD_ROOT modules_install
    # mark modules executable so that strip-to-file can strip them
    find $RPM_BUILD_ROOT/lib/modules/%{uname} -name "*.ko" -type f | xargs chmod u+x
    
    # Remove files created by depmod. These are recreated at install time.
    rm -f $RPM_BUILD_ROOT/lib/modules/%{uname}/modules.*
fi

# Save debuginfo
install -d -m 755 $RPM_BUILD_ROOT/usr/lib/debug/lib/modules/%{uname}
install -m 644 vmlinux $RPM_BUILD_ROOT/usr/lib/debug/lib/modules/%{uname}

# Install -headers files
make ${MAKEOPTS} INSTALL_HDR_PATH=$RPM_BUILD_ROOT/usr headers_install

# Install -devel files
install -d -m 755 $RPM_BUILD_ROOT/usr/src/kernels/%{uname}-%{target}

# Setup -devel links correctly
SOURCE=/usr/src/kernels/%{uname}-%{target}
rm -f $RPM_BUILD_ROOT/lib/modules/%{uname}/build
rm -f $RPM_BUILD_ROOT/lib/modules/%{uname}/source
ln -sf $SOURCE $RPM_BUILD_ROOT/lib/modules/%{uname}/source
ln -sf $SOURCE $RPM_BUILD_ROOT/lib/modules/%{uname}/build

install -d -m 755 $RPM_BUILD_ROOT${SOURCE}/arch/x86/kernel

cp --parents `find  -type f -name "Makefile*" -o -name "Kconfig*"` $RPM_BUILD_ROOT${SOURCE}
if [ -e Module.symvers ]; then
    install -m 644 Module.symvers $RPM_BUILD_ROOT${SOURCE}
fi
cp System.map $RPM_BUILD_ROOT${SOURCE}
if [ -s Module.markers ]; then
      cp Module.markers $RPM_BUILD_ROOT${SOURCE}
fi
# then drop all but the needed Makefiles/Kconfig files
rm -rf $RPM_BUILD_ROOT${SOURCE}/Documentation
rm -rf $RPM_BUILD_ROOT${SOURCE}/scripts
rm -rf $RPM_BUILD_ROOT${SOURCE}/include
install -m 644 arch/x86/kernel/asm-offsets.s $RPM_BUILD_ROOT${SOURCE}/arch/x86/kernel || :
install -m 644 .config $RPM_BUILD_ROOT${SOURCE}
install -m 644 .kernelrelease $RPM_BUILD_ROOT${SOURCE} || :
cp -a scripts $RPM_BUILD_ROOT${SOURCE}
cp -a arch/x86/scripts $RPM_BUILD_ROOT${SOURCE}/arch/x86 || :
cp -a arch/x86/*lds $RPM_BUILD_ROOT${SOURCE}/arch/x86 || :
rm -f $RPM_BUILD_ROOT${SOURCE}/scripts/*.o
rm -f $RPM_BUILD_ROOT${SOURCE}/scripts/*/*.o

if [ -d arch/x86/include ] ; then
    cp -a --parents arch/x86/include $RPM_BUILD_ROOT${SOURCE}
fi
if [ -d arch/x86/syscalls ] ; then
    cp -a --parents arch/x86/syscalls $RPM_BUILD_ROOT${SOURCE}
fi

cp -a include $RPM_BUILD_ROOT${SOURCE}/include

rm -f $RPM_BUILD_ROOT${SOURCE}/include/Kbuild
cp -a include/generated/uapi/linux/version.h $RPM_BUILD_ROOT${SOURCE}/include/linux/
cp -a include/generated/autoconf.h $RPM_BUILD_ROOT${SOURCE}/include/linux/

# Copy .config to include/config/auto.conf so "make prepare" is unnecessary.
cp $RPM_BUILD_ROOT${SOURCE}/.config $RPM_BUILD_ROOT${SOURCE}/include/config/auto.conf

# Make sure the Makefile, .config, auto.conf, autoconf.h and version.h have a matching 
# timestamp so that external modules can be built
touch -r $RPM_BUILD_ROOT${SOURCE}/Makefile $RPM_BUILD_ROOT${SOURCE}/.config
touch -r $RPM_BUILD_ROOT${SOURCE}/Makefile $RPM_BUILD_ROOT${SOURCE}/include/config/auto.conf
touch -r $RPM_BUILD_ROOT${SOURCE}/Makefile $RPM_BUILD_ROOT${SOURCE}/include/linux/autoconf.h
touch -r $RPM_BUILD_ROOT${SOURCE}/Makefile $RPM_BUILD_ROOT${SOURCE}/include/linux/version.h
touch -r $RPM_BUILD_ROOT${SOURCE}/Makefile $RPM_BUILD_ROOT${SOURCE}/include/generated/autoconf.h
touch -r $RPM_BUILD_ROOT${SOURCE}/Makefile $RPM_BUILD_ROOT${SOURCE}/include/generated/uapi/linux/version.h

# Firmware is provided by the linux-firmware package.
rm -rf $RPM_BUILD_ROOT/lib/firmware

find $RPM_BUILD_ROOT -name ..install.cmd -type f -delete

%clean
rm -rf $RPM_BUILD_ROOT

%post
exe=/sbin/new-kernel-pkg
if [ -x /sbin/new-kernel-pkg.py ] ; then
  exe=/sbin/new-kernel-pkg.py
fi
if grep -qs control_d /proc/xen/capabilities ; then
  args=--multiboot=/boot/xen.gz
fi

$exe --install --package kernel-xen --mkinitrd --depmod --make-default $args %{uname}

%preun
if [ "$1" = "0" ]; then
  exe=/sbin/new-kernel-pkg
  if [ -x /sbin/new-kernel-pkg.py ] ; then
    exe=/sbin/new-kernel-pkg.py
  fi
  $exe --remove --rminitrd --rmmoddep %{uname}
fi

%files
%defattr(-,root,root)
/boot/vmlinuz-%{uname}
/boot/System.map-%{uname}
/boot/config-%{uname}
%dir /lib/modules/%{uname}
/lib/modules/%{uname}/kernel

%files headers
%defattr(-,root,root)
/usr/include

%files devel
%defattr(-,root,root)
/lib/modules/%{uname}/build
/lib/modules/%{uname}/source
%verify(not mtime) /usr/src/kernels/%{uname}-%{target}

%changelog
* Wed Sep 25 2013 Malcolm Crossley <malcolm.crossley@eu.citrix.com> - 3.10
- Synced with elrepo kernel-ml packaging style
* Thu Mar 29 2012 Simon Rowe <simon.rowe@eu.citrix.com> - 3.3.0-rc3
- Packaged for XenServer
